#!/bin/sh
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

CURPATH=$(dirname $0)

. $CURPATH/initSC.sh

#Add the DB2 user ID to the TPCAdmins group
function updateGroupLinux
{
    DB2USER=$(grep rcs.dbuser $DIRHOME/lwi/conf/overrides/database.properties  | awk '{print $3}')

	echo "Checking if group exists..." >> $MIGLOG 2>&1
    #if group already exists then return success
    id -nG $DB2USER | grep TPCAdmin > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
       echo "Group already exists." >> $MIGLOG 2>&1
       return 0;
    fi
    
    echo "adding $DB2USER to TPCAdmin group " >> $MIGLOG 2>&1
    if [ -d $DIRHOME/vmi ] ; then
       usermod -A TPCAdmin $DB2USER
    else
       usermod -a -G TPCAdmin $DB2USER
    fi
    if [ $? -ne 0 ] ; then
      echo "Adding user to group failed with RC = $?" >> $MIGLOG 2>&1
      return 1
    fi
    
    echo "Adding user to group succeeded with RC = $?" >> $MIGLOG 2>&1
    return 0
}

#Add the DB2 user ID to the TPCAdmins group on AIX.
function updateGroupAIX
{
   echo "Checking if db2 user is part of the system group..." >> $MIGLOG 2>&1
   DB2USER=$(grep rcs.dbuser $DIRHOME/lwi/conf/overrides/database.properties  | awk '{print $3}')
   GROUPS=$(lsgroup -a -c users system |grep $DB2USER)
   if [ $? -ne 0 ] ; then
   	  echo "Adding db2 user to the system group..." >> $MIGLOG 2>&1
      CGROUPS=$(lsgroup -a -c users system |grep system | awk -F ':' '{print $2}')
      CGROUPS=$CGROUPS,$DB2USER
      chgroup users=$CGROUPS system
      if [ $? -ne 0 ] ; then
      	echo "Failed adding user to system group.  Return Code: $?" >> $MIGLOG 2>&1
      	return 1
      else
      	echo "Done adding user to system group.  Return Code: $?" >> $MIGLOG 2>&1
      	return 0
      fi
   else
      echo "db2 user is already part of the system group.  Return Code: $?" >> $MIGLOG 2>&1
      return 0 
   fi

}

#Helper function to determine which OS to call the update group for.
function updateGroup
{
   OS=$(uname)
   if [ "$OS" == "Linux" ]; then
      updateGroupLinux
      if [ $? -ne 0 ] ; then
         return 1
      fi
   else
#  Assume it is AIX
      updateGroupAIX
      if [ $? -ne 0 ] ; then
         return 1
      fi
   fi
}

#Start Storage Control
function startSC
{
   if [ -e $DIRHOME/StorageControl/bin/startStorageControl.sh ]; then
      $DIRHOME/StorageControl/bin/startStorageControl.sh >> $MIGLOG 2>&1 
   elif [ -e $DIRHOME/bin/startStorageControl.sh ]; then
      $DIRHOME/bin/startStorageControl.sh >> $MIGLOG 2>&1 
   fi
}

#Stop Storage Control
function stopSC
{
   if [ -e $DIRHOME/StorageControl/bin/stopStorageControl.sh ]; then
      $DIRHOME/StorageControl/bin/stopStorageControl.sh >> $MIGLOG 2>&1 
   elif [ -e $DIRHOME/bin/stopStorageControl.sh ]; then
      $DIRHOME/bin/stopStorageControl.sh >> $MIGLOG 2>&1 
   fi
}

MIGLOG=$DIRHOME/log/SCMigrate.log

#We need to stop Storage Control to support a bridge scenario.
#If Storage Control 4.2.1.1 or earlier is installed, but Director 6.3.1 or higher is installed 
#(Storage Control activation framework), then Storage Control will not automatically stop with Director.
#So we need to manually stop Storage Control in this case.  Otherwise, SC should already be stopped at this point.
stopSC

#Use the specified backup directory if one was given, otherwise use the default.
if [ "$1" != "" ] ; then
  BACKUP_DIR=$1
else
  BACKUP_DIR=/tmp/sc_backup
fi

BACKUPFILE=/tmp/sc_migrate_options.backup
RESTOREFILE=/tmp/sc_migrate_options.restore

#Execute SCMigrateDB.java with the specified parameters.
"$JAVAPATH/java" -DBACKUPDIR=$BACKUP_DIR -DBACKUPFILE=$BACKUPFILE -DRESTOREFILE=$RESTOREFILE -cp $CLASSPATH SCMigrateDB $DIRHOME $TPCHOME $* >> $MIGLOG 2>&1

if [ $? != 0 ] ; then
  displayMessage "MIGRATION_FAILED" >> $MIGLOG 2>&1
  # Remove the backup and restore files if they exist
  rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
  exit -5
fi

#If the backup file exists then run SCMigDBSupport.sh
if [ -f $BACKUPFILE ] ; then
  $DIRHOME/StorageControl/bin/SCMigDBSupport.sh $BACKUPFILE $DIRHOME
  if [ $? != 0 ] ; then
     displayMessage "MIGRATION_BACKUP_FAILED" >> $MIGLOG 2>&1
     # Remove the backup and restore files if they exist
     rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
     exit -8
  fi
else
  displayMessage "MIG_BACKUP_NOTFOUND" >> $MIGLOG 2>&1
  # Remove the backup and restore files if they exist
  rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
  exit -6
fi

#If the restore file exists, then run SCMigDBSupport.sh
if [ -f $RESTOREFILE ] ; then
  $DIRHOME/StorageControl/bin/SCMigDBSupport.sh $RESTOREFILE $DIRHOME
  if [ $? != 0 ] ; then
     displayMessage "MIGRATION_RESTORE_FAILED" >> $MIGLOG 2>&1
     # Remove the backup and restore files if they exist
     rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
     exit -9
  fi
else
  displayMessage "MIG_RESTORE_NOTFOUND" >> $MIGLOG 2>&1
  # Remove the backup and restore files if they exist
  rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
  exit -7
fi

#Delete the old TPCDB after a successful migration to Managed DB2
. $BACKUPFILE
echo "Dropping the TPCDB for instance owner $DB_USER..." >> $MIGLOG 2>&1
su - $DB_USER -c "db2 drop database TPCDB" >> $MIGLOG 2>&1
if [ $? -ne 0 ] ; then
	displayMessage "MIGRATION_FAILED" >> $MIGLOG 2>&1
	# Remove the backup and restore files if they exist
	rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
	exit 2
fi

#Run SCChangeDBCfg.sh
$DIRHOME/StorageControl/bin/SCChangeDBCfg.sh -u -p -v

if [ $? -eq 0 ] ; then
   displayMessage "MIGRATION_SUCCESS" >> $MIGLOG 2>&1
   # Remove the backup and restore files if they exist
   rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
   echo "Updating group" >> $MIGLOG 2>&1
   updateGroup
   if [ $? -ne 0 ] ; then
      echo "Failed updating group. Please adding $DB2USER manually to 'root' group" >> $MIGLOG 2>&1
   else
      echo "Starting SC Server" >> $MIGLOG 2>&1
   fi
   startSC
   if [ $? -ne 0 ] ; then
      echo "Failed starting SC Server. Please start manually." >> $MIGLOG 2>&1
   else
      echo "Started SC Server" >> $MIGLOG 2>&1
   fi
     
#  need to work around until we get the fix for hang
   sleep 30
   echo "Stopping SC Server" >> $MIGLOG 2>&1
   stopSC
   if [ $? -ne 0 ] ; then
      echo "Failed stopping SC Server. Please stop manually." >> $MIGLOG 2>&1
   else
      echo "Stopped SC Server" >> $MIGLOG 2>&1
   fi
   exit 0
else
   displayMessage "MIGRATION_FAILED" >> $MIGLOG 2>&1
   # Remove the backup and restore files if they exist
   rm -f $BACKUPFILE $RESTOREFILE >> $MIGLOG 2>&1
fi
